Attribute VB_Name = "modRouteMerge"
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' The code was written on 23-June-2002 by
' Chip Pearson, Pearson Software Consulting LLC,
' www.cpearson.com.
'
' It is explicitly granted to the Public Domain.
'
' Platform: VBA version 6 (any host application)
'
' External Dependencies:
'       MapPoint 2002 (version 9)
'       CWayPointX class.
'
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Private MPP As MapPoint.Application
Private Coll As Collection

Public Sub GetRoutes()
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' The procedure populates a Collection object with
' a set of CWayPointX objects, each of which represents
' a single WayPoint.
' Only the Name and latitude and longitude properties
' of the MapPoint WayPoint object are preserved.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Dim Rt As MapPoint.Route
Dim Mp As MapPoint.Map
Dim WP As MapPoint.Waypoint
Dim FName As String
Dim FNameArr As Variant
Dim Lat As Double
Dim Lng As Double
Dim Ndx As Long
Dim WPX As CWayPointX
'
' Get an existing instance of MapPoint, or
' create a new instance.
'
InitializeMapPointApp
'
' Here, we create a Variant Array of the file names
' that we want to open.  Note that files will be
' opened in the order in which they are in the array.
' Therefore, the waypoints of the "merged" route will
' be in the order in which the maps are opened. No sort
' of "start/end" checking is done.
'
FNameArr = Array( _
"C:\Maps\NewMexicoTrip\Day1_KC_To_Hereford.ptm", _
"C:\Maps\NewMexicoTrip\Day2_Hereford_To_Clovis.ptm", _
"C:\Maps\NewMexicoTrip\Day4_Clovis_To_Albeq.ptm", _
"C:\Maps\NewMexicoTrip\Day5_Albeq_VLA_To_Taos.ptm", _
"C:\Maps\NewMexicoTrip\Day6_Taos_To_CoSprings.ptm", _
"C:\Maps\NewMexicoTrip\Day7_CoSprings_To_Burlington.ptm", _
"C:\Maps\NewMexicoTrip\Day8_Burlington_To_KC.ptm")
'
' Clear and reinitialize the collection.
'
Set Coll = Nothing
Set Coll = New Collection
'
' Loop through the files in the array
'
For Ndx = LBound(FNameArr) To UBound(FNameArr)
    '
    ' Get the file name of the Map.
    '
    FName = FNameArr(Ndx)
    '
    ' open the mappoint file.
    '
    MPP.OpenMap Filename:=FName
    '
    ' Comment out the line of your host application doesn't
    ' support a StatusBar property.
    '
    Application.StatusBar = "Processing File: " & FName
    '
    ' get the Map and Route objects
    '
    Set Mp = MPP.ActiveMap
    Set Rt = Mp.ActiveRoute
    '
    ' Loop through the WayPoint collection for the Route
    '
    For Each WP In Rt.Waypoints
        Set WPX = New CWayPointX
        WPX.SetWayPoint WP
        '
        ' Here, we want to prevent duplicate WayPoints
        ' at the start of one route and the end of the
        ' previous route.  In other words, if route #1
        ' ends at the same location as route #2, we don't
        ' want two WayPoints for the same place, so we
        ' ignore the End waypoint of Route #1.
        '
        If WP.Type = geoWaypointEnd Then
            If Coll.Count > 0 Then
                If (WPX.Latitude = Coll(Coll.Count).Latitude) And _
                    (WPX.Longitude = Coll(Coll.Count).Longitude) Then
                    '
                    ' Duplicate. do nothing
                    '
                    Debug.Print "IGNORE: ", FName, WPX.Name
                Else
                    Coll.Add Item:=WPX
                End If
            Else
                Coll.Add Item:=WPX
            End If
        Else
            Coll.Add Item:=WPX
        End If
    Next WP
Next Ndx

MPP.Visible = True
'
' Comment out the line of your host application doesn't
' support a StatusBar property.
'
Application.StatusBar = False

    
End Sub


Public Sub CreateNewRouteMap()
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' The procedure creates a new Map and in that map creates a
' Route based upon all the CWayPointX objects in the Collection
' object Coll.  You must first run GetRoutes to populate the
' collection object.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Dim Rt As MapPoint.Route
Dim Mp As MapPoint.Map
Dim WP As MapPoint.Waypoint
Dim Loc As MapPoint.Location
Dim WPX As CWayPointX
Dim N As Long
'
' If our Collection object is Nothing, then
' we cannot recreate the routes.
'
If Coll Is Nothing Then
    MsgBox "Waypoint Collection object is has not been initialized" & vbCrLf & _
        "or has been reset. Unable to create routes.", vbOKOnly + vbExclamation
    Exit Sub
End If

Set Mp = MPP.NewMap
Set Rt = Mp.ActiveRoute
Rt.Clear
'
' Create the new Route object based on the waypoints represented by
' each CWayPointX object in the collection Coll.  Note, only
' the Name and latitude and longitude properties of a MapPoint
' WayPoint object are perserved in the CWayPointX object.
'
For Each WPX In Coll
    N = N + 1
    '
    ' Comment out the line of your host application doesn't
    ' support a StatusBar property.
    '
    Application.StatusBar = "Adding Waypoint: " & WPX.Name & _
        " (" & Format(N, "#,##0") & " of " & Format(Coll.Count) & ")"

    Set Loc = Mp.GetLocation(Latitude:=WPX.Latitude, Longitude:=WPX.Longitude)
    Rt.Waypoints.Add Anchor:=Loc, Name:=WPX.Name
Next WPX
'
' We have to calculate the route so it is rendered on
' the map.
'
Rt.Calculate
MPP.Visible = True
'
' Comment out the line of your host application doesn't
' support a StatusBar property.
'
Application.StatusBar = False

End Sub


Private Sub InitializeMapPointApp()
'
'  Initializes the global MPP object.
'
On Error Resume Next
'
' Comment out the line of your host application doesn't
' support a StatusBar property.
'
Application.StatusBar = "Initializing MapPoint Application."

If MPP Is Nothing Then
    ' grab an existing instance of MapPoint
    Set MPP = GetObject(, "MapPoint.Application")
    If Err.Number <> 0 Then
        Set MPP = New MapPoint.Application
    End If
End If
End Sub
